@font-face {
    font-family: roboto;
    src: url('../fonts/roboto.regular.ttf');
}

/* =======================================================================
inc_top.mako
========================================================================== */

.fa {
    margin-left: -2px;
    margin-right: 4px;
}

.navbaricon {
    margin-top: -15px;
    margin-bottom: -15px;
}

.icon-white {
    background-image: url("../images/glyphicons-halflings-white.png");
}

.ui-autocomplete-loading {
    background: white url("../images/loading16.gif") right center no-repeat;
}

.browserDialog.busy .ui-dialog-buttonpane {
    background: url("../images/loading.gif") 10px 50% no-repeat !important;
}

.ui-progressbar .ui-progressbar-overlay {
    background: url("../css/lib/images/animated-overlay.gif");
}

.ui-dialog,
.ui-dialog-buttonpane {
    background: #ECEADF url("../css/lib/images/ui-bg_fine-grain_10_eceadf_60x60.png") 50% 50% repeat !important;
}

/* restore 1.8.x resize handle on dialog button pane */
.ui-dialog .ui-resizable-se {
    width: 14px;
    height: 14px;
    right: 3px;
    bottom: 3px;
    background-position: -80px -224px;
}

.ui-accordion-content,
.ui-tabs-panel {
    background: #EDEDED none !important;
}

.ui-widget {
    font-family: inherit;
    font-size: 1em;
}

.ui-widget-content {
    background: #DCDCDC url("../css/lib/images/ui-bg_highlight-soft_75_dcdcdc_1x100.png") 50% top repeat-x;
}

.ui-widget-header {
    background: #FFFFFF url("../css/lib/images/ui-bg_flat_0_ffffff_40x100.png") 50% 50% repeat-x;
}

.ui-state-default,
.ui-widget-content .ui-state-default,
.ui-widget-header .ui-state-default {
    background: #FFFFFF;
    border: 1px solid #CCCCCC;
}

.ui-state-hover,
.ui-widget-content .ui-state-hover,
.ui-widget-header .ui-state-hover,
.ui-state-focus,
.ui-widget-content .ui-state-focus,
.ui-widget-header .ui-state-focus {
    background: #FFFFFF;
}

.ui-state-active,
.ui-widget-content .ui-state-active,
.ui-widget-header .ui-state-active {
    background: #F7F7F7;
}

.ui-state-highlight,
.ui-widget-content .ui-state-highlight,
.ui-widget-header .ui-state-highlight {
    background: #FBF9EE url("../css/lib/images/ui-bg_glass_55_fbf9ee_1x400.png") 50% 50% repeat-x;
}

.ui-state-error,
.ui-widget-content .ui-state-error,
.ui-widget-header .ui-state-error {
    background: #FEF1EC url("../css/lib/images/ui-bg_glass_95_fef1ec_1x400.png") 50% 50% repeat-x;
}

.ui-icon {
    float: left;
}

.ui-icon,
.ui-widget-content .ui-icon {
    background-image: url("../css/lib/images/ui-icons_222222_256x240.png");
}

.ui-widget-header .ui-icon {
    background-image: url("../css/lib/images/ui-icons_222222_256x240.png");
}

.ui-state-default .ui-icon {
    background-image: url("../css/lib/images/ui-icons_8c291d_256x240.png");
}

.ui-state-hover .ui-icon,
.ui-state-focus .ui-icon {
    background-image: url("../css/lib/images/ui-icons_222222_256x240.png");
}

.ui-state-active .ui-icon {
    background-image: url("../css/lib/images/ui-icons_8c291d_256x240.png");
}

.ui-state-highlight .ui-icon {
    background-image: url("../css/lib/images/ui-icons_2e83ff_256x240.png");
}

.ui-state-error .ui-icon,
.ui-state-error-text .ui-icon {
    background-image: url("../css/lib/images/ui-icons_cd0a0a_256x240.png");
}

.ui-widget-overlay {
    background: #AAAAAA url("../css/lib/images/ui-bg_flat_0_aaaaaa_40x100.png") 50% 50% repeat-x;
}

.ui-widget-shadow {
    background: #000000 url("../css/lib/images/ui-bg_flat_0_000000_40x100.png") 50% 50% repeat-x;
}

.ui-state-active a, .ui-state-active a:link, .ui-state-active a:visited {
    color: #140F06;
    text-decoration: none;
}

.ui-state-default a, .ui-state-default a:link, .ui-state-default a:visited {
    color: #222222;
    text-decoration: none;
}

.ui-tabs {
    padding: 0;
    background: none;
    border-width: 0;
}

.ui-tabs .ui-tabs-nav {
    padding-left: 0;
    background: transparent;
    border-width: 0;
    -moz-border-radius: 0;
    -webkit-border-radius: 0;
    border-radius: 0;
}

.ui-tabs .ui-tabs-panel {
    background-color: #F7F7F7 !important;
    border: 1px solid #CCCCCC !important;
    padding: 1em;
}

.ui-tabs .ui-tabs-nav li.ui-tabs-active {
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
}

.ui-tabs-nav > :not(.ui-tabs-active) {
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
}

#sub-menu-container {
    margin: 0;
}

#sub-menu {
    clear: both;
    font-size: 12px;
    align-content: flex-end;
}

#sub-menu a {
    clear-after: top;
    margin: 2px;
    float: right;
}

.upgrade-notification {
    max-width: 600px;
    text-align: center;
    margin: 10px auto;
}

[class^="menu-icon-"], [class*=" menu-icon-"] {
    background: url("../images/menu/16px_sprite_colored_menu_icons.png");
    height: 16px;
    width: 16px;
    display: inline-block;
    position: relative;
    right: 3px;
    top: 0;
    float: left;
}

.navbar [class^="menu-icon-"], .navbar [class*=" menu-icon-"] {
    top: 1px;
}

.menu-icon-addshow {
    background-position: 0 0;
}

.menu-icon-anime {
    background-position: -21px 0;
}

.menu-icon-backlog-view {
    background-position: -42px 0;
}

.menu-icon-backlog {
    background-position: -63px 0;
}

.menu-icon-bittorrent {
    background-position: -84px 0;
}

.menu-icon-config-index {
    background-position: -105px 0;
}

.menu-icon-config {
    background-position: -126px 0;
}

.menu-icon-failed-download {
    background-position: -147px 0;
}

.menu-icon-home {
    background-position: -168px 0;
}

.menu-icon-manage {
    background-position: -189px 0;
}

.menu-icon-manage-searches {
    background-position: -210px 0;
}

.menu-icon-poster {
    background-position: -231px 0;
}

.menu-icon-postprocess {
    background-position: -252px 0;
}

.menu-icon-restart {
    background-position: -273px 0;
}

.menu-icon-shutdown {
    background-position: -294px 0;
}

.menu-icon-update {
    background-position: -315px 0;
}

.menu-icon-viewlog-errors {
    background-position: -336px 0;
}

.menu-icon-viewlog {
    background-position: -357px 0;
}

.menu-icon-kodi {
    background-position: -378px 0;
}

.menu-icon-help {
    background-position: -399px 0;
}

.menu-icon-info {
    background-position: -418px 0;
}

.menu-icon-error {
    background-position: -436px 0;
}

.menu-icon-news {
    background-position: -456px 0;
}

.menu-icon-irc {
    background-position: -478px 0;
}

.menu-icon-changelog {
    background-position: -495px 0;
}

.menu-icon-support {
    background-position: -516px 0;
}

.menu-icon-plex {
    background-position: -536px 0;
}

.menu-icon-backup {
    background-position: -556px 0;
}

.menu-icon-provider {
    background-position: -576px 0;
}

.menu-icon-notification {
    background-position: -597px 0;
}

.menu-icon-emby {
    background-position: -614px 0;
}

.menu-icon-blackhole {
    background-position: -632px 0;
}

.menu-icon-schedule {
    background-position: -653px 0;
}

.menu-icon-manage2 {
    background-position: -673px 0;
}

.menu-icon-history {
    background-position: -695px 0;
}

.menu-icon-trash {
    background-position: -711px 0;
}

.menu-icon-cut {
    background-position: -727px 0;
}

.menu-icon-select {
    background-position: -742px 0;
}

/* =======================================================================
inc_bottom.mako
========================================================================== */

.footer {
    padding: 20px 0;
    color: #4E4E4E;
    text-align: center;
    font-size: 12px;
}

.footerhighlight {
    color: #111111;
    display: inline;
}

.footer-item {
    white-space: nowrap;
}

/* =======================================================================
inc_rootDirs.mako
========================================================================== */
.rootdir-selectbox,
.rootdir-selectbox #rootDirs,
.rootdir-controls {
    width: 100%;
}

.rootdir-selectbox {
    padding: 0 0 5px;
}

.rootdir-controls {
    text-align: center;
}

/* =======================================================================
home.mako
========================================================================== */

.imgbanner .banner {
    border: 1px solid #CCCCCC;
    overflow: hidden;
    height: 66px;
    border-radius: 8px;
    vertical-align: top;
    width: 360px;
    display: block;
    margin-left: auto;
    margin-right: auto;
}

.imgsmallposter .small {
    height: 66px;
    overflow: hidden;
    border-radius: 3px;
    vertical-align: middle;
    width: 45px;
    border: 1px solid #CCCCCC;
    margin-right: 5px;
}

.search {
    margin-bottom: 10px;
}

.ui-progressbar {
    height: 20px;
    line-height: 18px;
}

.ui-progressbar .ui-progressbar-value {
    box-sizing: content-box !important;
}

.progressbarText {
    position: absolute;
    top: 0;
    width: 100%;
    height: 100%;
    overflow: visible;
    text-align: center;
    text-shadow: 0 0 0.1em #FFFFFF;
    vertical-align: middle;
    font-size: 12px;
    color: #000000;
}

.progress-100 {
    background-image: -moz-linear-gradient(#A6CF41, #5B990D);
    background-image: linear-gradient(#A6CF41, #5B990D);
    background-image: -webkit-linear-gradient(#A6CF41, #5B990D);
    background-image: -o-linear-gradient(#A6CF41, #5B990D);
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px;
}

.progress-80 {
    background-image: -moz-linear-gradient(#E1FF97, #9DB269);
    background-image: linear-gradient(#E1FF97, #9DB269);
    background-image: -webkit-linear-gradient(#E1FF97, #9DB269);
    background-image: -o-linear-gradient(#E1FF97, #9DB269);
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px;
}

.progress-60 {
    background-image: -moz-linear-gradient(#FAD440, #F2A70D);
    background-image: linear-gradient(#FAD440, #F2A70D);
    background-image: -webkit-linear-gradient(#FAD440, #F2A70D);
    background-image: -o-linear-gradient(#FAD440, #F2A70D);
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px;
}

.progress-40 {
    background-image: -moz-linear-gradient(#FAB543, #F2700D);
    background-image: linear-gradient(#FAB543, #F2700D);
    background-image: -webkit-linear-gradient(#FAB543, #F2700D);
    background-image: -o-linear-gradient(#FAB543, #F2700D);
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px;
}

.progress-20 {
    background-image: -moz-linear-gradient(#DA5945, #B11A10);
    background-image: linear-gradient(#DA5945, #B11A10);
    background-image: -webkit-linear-gradient(#DA5945, #B11A10);
    background-image: -o-linear-gradient(#DA5945, #B11A10);
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px;
}

span.show-option,
span.movie-option {
    white-space: nowrap;
    margin-left: 10px;
    line-height: 40px;
}

.show-grid, .movie-grid {
    display: none;
}

.poster-container {
    padding-right: 10px;
}

.info-container {
    overflow: hidden;
}

#container, #container-anime {
    margin: 0 auto;
}

.loading-spinner {
    background-image: url("../images/loading32.gif");
    width: 32px;
    height: 32px;
    display: inline-block;
}

.loading-spinner16 {
    background-image: url("../images/loading16.gif");
    width: 16px;
    height: 16px;
    display: inline-block;
    position: relative;
    top: 3px;
}

.show-container, .movie-container {
    margin: 4px;
    background-color: #F3F3F3;
    border: 5px solid #F3F3F3;
    overflow: hidden;
    box-shadow: 1px 1px 3px 0 rgba(0, 0, 0, 0.31);
}

.show-details, .movie-details {
    height: 32px;
    overflow: hidden;
}

.show-image, .movie-image {
    overflow: hidden;
    max-width: 100%;
}

.show-container .ui-progressbar, .movie-container .ui-progressbar {
    height: 7px !important;
    top: -2px;
}

.show-container .ui-corner-all, .ui-corner-bottom, .ui-corner-right, .ui-corner-br,
.movie-container .ui-corner-all, .ui-corner-bottom, .ui-corner-right, .ui-corner-br {
    border-bottom-right-radius: 0;
}

.show-container .ui-corner-all, .ui-corner-bottom, .ui-corner-left, .ui-corner-bl,
.movie-container .ui-corner-all, .ui-corner-bottom, .ui-corner-left, .ui-corner-bl {
    border-bottom-left-radius: 0;
}

.show-container .ui-corner-all, .ui-corner-top, .ui-corner-right, .ui-corner-tr,
.movie-container .ui-corner-all, .ui-corner-top, .ui-corner-right, .ui-corner-tr {
    border-top-right-radius: 0;
}

.show-container .ui-corner-all, .ui-corner-top, .ui-corner-left, .ui-corner-tl,
.movie-container .ui-corner-all, .ui-corner-top, .ui-corner-left, .ui-corner-tl {
    border-top-left-radius: 0;
}

.show-container .ui-widget-content,
.movie-container .ui-widget-content {
    border-top: 1px solid #111111;
    border-bottom: 1px solid #111111;
    border-left: 0;
    border-right: 0;
}

.ui-progressbar .ui-progressbar-value {
    height: 20px
}

.ui-progressbar .progress-20 {
    border: none;
}

.show-container .progress-20,
.show-container .progress-40,
.show-container .progress-60,
.show-container .progress-80,
.movie-container .progress-20,
.movie-container .progress-40,
.movie-container .progress-60,
.movie-container .progress-80 {
    border-radius: 0;
    height: 7px
}

.show-title, .movie-title {
    position: relative;
    overflow: hidden;
    white-space: nowrap;
    font-size: 11px;
    margin: 4px 4px 0 4px;
}

.show-title:after, .movie-title:after {
    content: "";
    pointer-events: none;
    position: absolute;
    width: 20px;
    height: 100%;
    top: 0;
    right: 0;
    background-image: -webkit-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: -moz-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: -ms-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: -o-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: linear-gradient(to left, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
}

.show-date, .movie-date {
    position: relative;
    overflow: hidden;
    white-space: nowrap;
    font-size: 11px;
    margin: 0 4px 4px 4px;
    color: #949494;
}

.show-date:after, .movie-date:after {
    content: "";
    pointer-events: none;
    position: absolute;
    width: 20px;
    height: 100%;
    top: 0;
    right: 0;
    background-image: -webkit-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: -moz-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: -ms-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: -o-linear-gradient(right, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
    background-image: linear-gradient(to left, rgba(243, 243, 243, 1), rgba(243, 243, 243, 0));
}

.show-table, .movie-table {
    text-align: center;
    vertical-align: middle;
    width: 33%
}

.show-add, .movie-add {
    font-size: 11px;
    text-align: left;
    display: block;
}

.show-status, .movie-status {
    font-size: 11px;
    text-align: left;
    display: block;
}

.show-network-image, .movie-network-image {
    width: 50px;
    height: auto;
}

.show-dlstats, .movie-dlstats {
    font-size: 11px;
    text-align: left;
    display: block;
    margin-left: 4px;
}

.show-quality, .movie-quality {
    font-size: 11px;
    text-align: right;
    display: block;
    margin-right: 4px;
}

#sort-by {
    display: inline;
    list-style-type: none;
    padding: 0;
    margin-left: 5px;
}

#sort-by ul, #sort-by li {
    display: inline;
    margin: 0;
    padding: 0;
}

.posterview {
    margin: 0 auto;
}

td.tvShow a {
    color: #000000;
    text-decoration: none;
}

td.tvShow a:hover {
    cursor: pointer;
    color: #428BCA;
}

#popover-target label {
    margin: 0 5px;
    display: block;
}

#popover-target input {
    margin-right: 5px;
}

.popover {
    margin-left: -48px;
}

/* =======================================================================
home_addShows.mako
========================================================================== */

#addShowPortal {
    max-width: 700px;
    margin: 15px auto;
}

#addShowPortal a {
    padding: 10px;
}

div.button {
    display: table-cell;
    vertical-align: middle;
    padding-left: 10px;
}

div.buttontext {
    display: table-cell;
    padding-left: 20px;
    text-align: left;
    white-space: normal;
}

div.buttontext h3 {
    margin-top: 10px;
}

div.buttontext p {
    font-size: 13px;
}

[class^="add-list-icon-"], [class*=" add-list-icon-"] {
    background: url("../images/32px_sprite_colored_add_lists.png");
    height: 32px;
    width: 32px;
    display: inline-block;
    position: relative;
    top: 2px;
    float: left;
}

.add-list-icon-addnewshow {
    background-position: 0 0;
}

.add-list-icon-addtrakt {
    background-position: -38px 0;
}

.add-list-icon-addimdb {
    background-position: -76px 0;
}

add-list-icon-addtvdb {
    background-position: 0px 0;
}

.add-list-icon-addexistingshow {
    background-position: -113px 0;
}

.add-list-icon-addanime {
    background-position: -150px 0;
}

/* =======================================================================
home_newShow.mako
========================================================================== */

#addShowForm, #recommendedShowsForm {
    font-size: 13px;
}

#displayText {
    padding: 8px;
    overflow: hidden;
    font-size: 14px;
    background-color: #EFEFEF;
    border: 1px solid #DFDEDE;
}

#searchResults {
    height: 100%;
}

#searchResults .legendStep {
    margin-bottom: 5px;
}

#searchResults input[type="radio"] {
    vertical-align: -2px;
}

/* =======================================================================
home_addExistingShow.mako
========================================================================== */

.existingtabs {
    padding: 1em 1.4em;
}

ul#rootDirStaticList {
    margin-right: auto;
    margin-left: auto;
    text-align: left;
    padding-left: 0;
}

ul#rootDirStaticList li {
    padding: 4px 5px 4px 5px;
    margin: 2px;
    list-style: none outside none;
    background: url('../css/lib/images/ui-bg_highlight-soft_75_efefef_1x100.png') repeat-x scroll 50% 50% #EFEFEF;
}

ul#rootDirStaticList li label {
    margin-top: 5px;
    margin-bottom: 5px;
    cursor: pointer;
}

ul#rootDirStaticList li input[type="checkbox"] {
    vertical-align: -3px;
}

/* =======================================================================
home_trendingShows.mako
========================================================================== */

.traktShowTitleIcons {
    float: right;
    padding-right: 4px;
    padding-bottom: 4px;
}

.traktContainer p {
    padding-top: 2px;
}

.traktContainer p img {
    position: relative;
    top: -2px;
}

.traktContainer p, .traktContainer i {
    white-space: nowrap;
    font-size: 12px;
    overflow: hidden;
    padding-left: 4px;
    margin: 0;
}

.traktContainer {
    margin: 12px;
    width: 188px;
    background-color: #DFDACF;
    border: 1px solid #111111;
    border-radius: 6px;
}

.trakt-image, .trakt-image img {
    height: 273px;
    width: 186px;
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
    border-bottom: 1px solid #111111;
}

.trakt-image img {
    background-image: url(../images/trakt-placeholder.png);
}

/* =======================================================================
displayShow.mako
========================================================================== */

#pickShow {
    display: inline-block;
    width: calc(100% - 40px);
}

.poster-container {
    float:left;
}

#posterCol {
    float: left;
    margin-right: 10px;
    margin-bottom: 20px;
}

.navShow {
    display: inline;
    cursor: pointer;
}

#prevShow,
#nextShow,
#topcontrol {
    -webkit-filter: grayscale(100%);
    filter: url("data:image/svg+xml;utf8,<svg version='1.1' xmlns='http://www.w3.org/2000/svg' height='0'><filter id='greyscale'><feColorMatrix type='matrix' values='0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0 0 0 1 0' /></filter></svg>#greyscale");
}

h1.title {
    padding-bottom: 12px;
    margin-bottom: 15px;
    line-height: 30px;
    text-align: left;
    text-rendering: optimizelegibility;
    border-bottom: 1px solid #888888;
}

.displayspecials {
    position: relative;
    top: -24px;
}

#showinfo {
    cursor: default;
}

#showinfo {
    display: inline-block;
    position: relative;
}

span.imdbstars {
    display: inline-block;
    vertical-align: top;
    cursor: help;
    margin-top: 4px;
}

span.imdbstars, span.imdbstars > * {
    height: 12px;
    background: url(../images/rating.png) 0 -12px repeat-x;
    width: 120px;
    display: inline-block;
    vertical-align: top;
}

span.imdbstars > * {
    background-position: 0 0;
    max-width: 120px;
}

ul.tags {
    list-style-type: none;
    position: relative;
    top: 4px;
    margin-left: -40px;
}

ul.tags li {
    margin-right: 4px;
    margin-bottom: 5px;
    padding: 3px 4px 3px 25px;
    background: url(../images/tag.png) no-repeat scroll 5px 4px #555555;
    border-radius: 3px;
    border: 1px solid #111111;
    color: #FFFFFF;
    font: 14px/18px roboto, "Open Sans", "Helvetica Neue", Helvetica, Arial, Geneva, sans-serif;
    text-shadow: 0 1px rgba(0, 0, 0, 0.8);
    float: left;
}

ul.tags li a {
    color: #FFFFFF;
}

.tvshowImg, .movieImg {
    border: 1px solid #CCCCCC;
    border-radius: 5px;
    max-height: 311px;
    margin: 0 auto;
}

#summary {
    margin-top: 5px;
    margin-bottom: 5px;
    padding: 10px;
    background-color: #EFEFEF;
    border: 1px solid #DFDEDE;
    width: 100%;
    overflow: auto;
    cursor: default;
    opacity: 0.8;
}

.summaryTable {
    overflow: hidden;
}

#checkboxControls {
    padding-top: 5px;
}

#checkboxControls span {
    padding: 5px;
    font-size: 13px;
    font-weight: bold;
    border-radius: 5px;
}

#checkboxControls label {
    white-space: nowrap;
    display: inline-block;
    margin: 7px 2px;
}

.unaired {
    background-color: #F5F1E4;
}

.skipped {
    background-color: #BEDEED;
}

.good {
    background-color: #C3E3C8;
}

.qual {
    background-color: #FFDA8A;
}

.wanted {
    background-color: #FFB0B0;
}

.snatched {
    background-color: #EBC1EA;
}

span.unaired {
    color: #584B20;
    border: 1px solid #584B20;
}

span.skipped {
    color: #1D5068;
    border: 1px solid #1D5068;
}

span.good {
    color: #295730;
    border: 1px solid #295730;
}

span.qual {
    color: #765100;
    border: 1px solid #765100;
}

span.wanted {
    color: #890000;
    border: 1px solid #890000;
}

span.snatched {
    color: #652164;
    border: 1px solid #652164;
}

span.unaired b,
span.skipped b,
span.good b,
span.qual b,
span.wanted b,
span.snatched b {
    color: #000000;
    font-weight: 800;
}

.plotInfo {
    cursor: help;
    float: right;
    position: relative;
    top: 2px;
}

.plotInfoNone {
    cursor: help;
    float: right;
    position: relative;
    top: 2px;
    opacity: 0.4;
}

.displayShowTable {
    table-layout: auto;
    width: 100%;
    border-collapse: collapse;
    border-spacing: 0;
    text-align: center;
    border: none;
    empty-cells: show;
    color: #000000 !important;
}

.displayShowTable.display_show {
    clear: both;
    opacity: 0.8;
}

.displayShowTable th.row-seasonheader {
    border: none !important;
    background-color: #FFFFFF;
    color: #FFFFFF !important;
    padding-top: 15px !important;
    text-align: left !important;
}

.displayShowTable td.col-search {
    text-align: center;
}

.sickchillTable {
    table-layout: auto;
    width: 100%;
    border-collapse: collapse;
    border-spacing: 0;
    text-align: center;
    border: none;
    empty-cells: show;
    padding: 0;
}

.sickchillTable.display_show {
    clear: both
}

.sickchillTable th {
    color: #FFFFFF;
    text-align: center;
    background-color: #333333;
    white-space: nowrap;
}

.sickchillTable th,
.sickchillTable td {
    border-top: 1px solid #FFFFFF;
    border-left: 1px solid #FFFFFF;
    padding: 4px;
}

th.row-seasonheader {
    border: none;
    background-color: #FFFFFF;
    color: #000000;
    padding-top: 15px;
    text-align: left;
}

tr.seasonheader {
    padding-bottom: 5px;
    padding-top: 10px;
    text-align: left;
    border: none;
    color: #FFFFFF;
}

th.col-checkbox,
td.col-checkbox {
    width: 30px;
    border-left: none;
    text-align: center;
}

th.col-checkbox input[type="checkbox"],
td.col-checkbox input[type="checkbox"] {
    vertical-align: -2px;
}

th.col-metadata,
td.col-metadata {
    width: 28px;
}

th.col-ep,
td.col-ep {
    width: 50px;
    white-space: nowrap;
}

th.col-airdate,
td.col-airdate {
    width: 86px;
    white-space: nowrap;
}

th.col-name,
td.col-name {
    min-width: 100px;
}

td.col-name {
    text-align: left;
}

th.col-network,
td.col-network {
    width: 95px;
}

th.col-subtitles,
td.col-subtitles {
    width: 150px;
    text-align: center;
}

th.col-status,
td.col-status {
    width: 230px;
    text-align: center;
}

th.col-quality,
td.col-quality {
    width: 110px;
}

th.col-legend,
td.col-legend {
    width: 80px;
}

th.col-search,
td.col-search {
    width: 46px;
}

.showLegend, .movieLegend {
    padding-right: 6px;
    padding-bottom: 1px;
    width: 150px;
}

.input-scene {
    height: 20px;
    line-height: 1.5;
    border-radius: 3px;
}

#editShow {
    width: 700px;
    padding-top: 10px;
    margin-right: auto;
    margin-left: auto;
}

#customize .custom-image {
    cursor: pointer;
}

#customize .custom-image.modified {
    box-shadow: 3px 3px green,
         3px -3px green,
         -3px 3px green,
         -3px -3px green;
}

[class^="displayshow-icon-"], [class*=" displayshow-icon-"] {
    background: url("../images/16px_sprite_colored_show.png");
    height: 16px;
    width: 16px;
    display: inline-block;
    position: relative;
    top: 3px;
}

.displayshow-icon-left {
    background-position: 0 0;
}

.displayshow-icon-right {
    background-position: -18px 0;
}

.displayshow-icon-imdb {
    background-position: -37px 0;
}

.displayshow-icon-tvdb {
    background-position: -56px 0;
}

.displayshow-icon-tvrage {
    background-position: -76px 0;
}

.displayshow-icon-tvmaze {
    background-position: -95px 0;
}

.displayshow-icon-sportdb {
    background-position: -113px 0;
}

.displayshow-icon-tmdb {
    background-position: -129px 0;
}

.displayshow-icon-anime {
    background-position: -150px 0;
}

.displayshow-icon-xem {
    background-position: -170px 0;
}

.displayshow-icon-disable {
    background-position: -190px 0;
}

.displayshow-icon-enable {
    background-position: -206px 0;
}

.displayshow-icon-sub {
    background-position: -224px 0;
}

.displayshow-icon-search {
    background-position: -243px 0;
}

.displayshow-icon-info {
    background-position: -261px 0;
}

.displayshow-icon-heart {
    background-position: -278px 0;
}

.displayshow-icon-legend {
    background-position: -297px 0;
    margin-top: 10px;
}

.displayshow-icon-warning {
    background-position: -316px 0;
}

.displayshow-icon-tag {
    background-position: -333px 0;
}

.displayshow-icon-clock {
    background-position: -350px 0;
}

.displayshow-icon-minus {
    background-position: -369px 0;
}

.displayshow-icon-plus {
    background-position: -387px 0;
}

.displayshow-icon-good {
    background-position: -403px 0;
}

.displayshow-icon-bad {
    background-position: -421px 0;
}

.displayshow-icon-fanart {
    background-position: -439px 0;
}

.displayshow-icon-trakt {
    background-position: -456px 0;
}

/* =======================================================================
schedule.mako
========================================================================== */

.key {
    line-height: 30px;
}

.listing-key {
    padding: 5px;
    font-size: 13px;
    font-weight: bold;
    border-radius: 5px;
}

.listing-default {
    background-color: #F5F1E4;
}

.listing-current {
    background-color: #DDFFDD;
}

.listing-overdue {
    background-color: #FFDDDD;
}

.listing-snatched {
    background-color: #EBC1EA;
}

.listing-toofar {
    background-color: #BEDEED;
}

span.listing-default {
    color: #826F30;
    border: 1px solid #826F30;
}

span.listing-current {
    color: #295730;
    border: 1px solid #295730;
}

span.listing-overdue {
    color: #890000;
    border: 1px solid #890000;
}

span.listing-snatched {
    color: #652164;
    border: 1px solid #652164;
}

span.listing-toofar {
    color: #1D5068;
    border: 1px solid #1D5068;
}

h2.day, h2.network {
    margin: 0 0 10px 0;
    font-size: 24px;
    line-height: 36px;
    font-weight: bold;
    letter-spacing: 1px;
    color: #FFFFFF;
    text-align: center;
    text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.3);
    background-color: #333333;
}

.tvshowDiv {
    display: block;
    clear: both;
    border: 1px solid #CCCCCC;
    margin: auto;
    padding: 0;
    text-align: left;
    width: 100%;
    border-radius: 5px;
    background: #FFFFFF;
    cursor: default;
    overflow: hidden;
}

.tvshowDiv a:hover {
    color: #428BCA;
}

.tvshowDiv a, .tvshowDiv a:link, .tvshowDiv a:visited, .tvshowDiv a:hover {
    text-decoration: none;
    background: none;
}

.tvshowTitle a {
    color: #000000;
    float: left;
    line-height: 1.4em;
    font-size: 1.4em;
    text-shadow: -1px -1px 0 #FFFFFF;
}

.tvshowTitleIcons {
    float: right;
    padding: 3px 5px;
}

.tvshowDiv td {
    padding: 5px 10px;
}

.tvshowDiv td.next_episode {
    width: 100%;
    height: 90%;
    border-bottom: 1px solid #CCCCCC;
    vertical-align: top;
    color: #000000;
}

.bannerThumb {
    vertical-align: top;
    height: auto;
    width: 100%;
    border-bottom: 1px solid #CCCCCC;
}

.posterThumb {
    vertical-align: top;
    height: auto;
    width: 180px;
    border-right: 1px solid #CCCCCC;
}

.ep_listing {
    width: auto;
    border: 1px solid #CCCCCC;
    margin-bottom: 10px;
    padding: 10px;
}

.ep_summary {
    margin-left: 5px;
    font-style: italic;
}

.ep_summaryTrigger {
    cursor: pointer;
    vertical-align: middle;
}

.ep_summaryTriggerNone {
    opacity: 0.4;
    vertical-align: middle;
}

.calendarWrapper {
    width: 1000px;
    margin: 0 auto;
    padding: 0 3px
}

.calendarTable {
    float: left;
    width: 142px !important;
    white-space: nowrap;
    table-layout: fixed !important;
}

.calendarShow {
    padding: 0 !important
}

.calendarShow .poster {
    padding-bottom: 2px
}

.calendarShow .poster img {
    width: 142px;
    height: auto
}

.calendarShow .text {
    padding: 0 5px 10px 5px
}

.calendarShow .text .airtime,
.calendarShow .text .episode-title {
    overflow: hidden;
    text-overflow: ellipsis;
    display: block;
    font-size: 11px
}

.calendarShow .show-status,
.calendarShow .movie-status {
    padding: 5px 10px 10px;
    text-align: center
}

table.cal-odd {
    background-color: #DDDDDD;
}

table.cal-even {
    background-color: #D2D2D2;
}

.calendarShow .text .airtime {
    color: #000000
}

.calendarShow .text .episode-title {
    color: #888888
}

/* =======================================================================
config*.mako
========================================================================== */

#config-content {
    display: block;
    width: 960px;
    padding: 0 0 40px;
    margin: 0 auto;
    clear: both;
    text-align: left;
}

#config input[type=checkbox] ~ label {
    padding-top: 4px;
}

.config-group-divider {
    margin: 20px 0;
    border-bottom: 1px dotted #CCCCCC;
}

.component-group {
    padding: 25px 0;
    border-bottom: 1px dotted #CCCCCC;
}

.component-item {
    border-bottom: 1px dotted #666666;
    min-height: 200px;
}

.component-group-desc {
    float: left;
}

.component-group-desc h3 {
    margin-top: 5px;
}

.component-group-desc p {
    width: 90%;
    margin: 10px 0;
    color: #666666;
}

#config div.field-pair {
    padding: 12px 0;
}

#config div.field-pair select,
#config div.field-pair input {
    margin-right: 6px;
}

#config div.field-pair input {
    float: left;
}

#config label.component-title,
#config span.component-title {
    float: left;
    margin-right: 10px;
    font-size: 13px;
    font-weight: bold;
}

#config .component-desc {
    font-size: 12px;
    font-weight: normal;
    display: block;
}

#config label.space-right {
    margin-right: 10px
}

#config .metadataDiv {
    display: none;
}

.component-group-save {
    float: right;
    padding-top: 10px;
}

select .selected {
    font-weight: 700;
}

.jumbo {
    font-size: 15px !important;
    line-height: 24px;
}

.testNotification {
    padding: 5px;
    margin-bottom: 10px;
    line-height: 20px;
    border: 1px dotted #CCCCCC;
}

#providerOrderList {
    width: 250px;
    padding-left: 20px;
    list-style-type: none;
}

#provider_order_list,
#service_order_list {
    width: 250px;
    padding-left: 20px;
    list-style-type: none;
}

#provider_order_list li,
#service_order_list li {
    padding: 5px;
    margin: 5px 0;
    font-size: 14px;
}

#provider_order_list .ui-state-default.torrent-provider {
    background-color: #FFFFFF !important;
}

#provider_order_list .ui-state-default.nzb-provider {
    background-color: #DDDDDD !important;
}

#provider_order_list input,
#service_order_list input {
    margin: 0 2px;
}

#config .tip_scale label span.component-title {
    width: 85px !important;
    font-size: 12px !important;
    margin-top: 2px !important;
}

#config .tip_scale label span.component-desc {
    margin-left: 120px !important;
    width: 220px !important;
}

.infoTableHeader,
.infoTableCell {
    padding: 5px;
}

.infoTableSeperator {
    border-top: 1px dotted #666666;
}

.infoTableHeader {
    vertical-align: top;
}

[class^="icon16-config-"], [class*=" icon16-config-"] {
    background: url("../images/16px_sprite_colored_help-info.png") no-repeat -40px 0;
    display: inline-block;
    height: 16px;
    width: 16px;
    line-height: 16px;
    vertical-align: text-top;
}

.icon16-config-sickchill {
    background-position: 0 0;
}

.icon16-config-version {
    background-position: -17px 0;
}

.icon16-config-python {
    background-position: -37px 0;
}

.icon16-config-ssl {
    background-position: -56px 0;
}

.icon16-config-locale {
    background-position: -73px 0;
}

.icon16-config-user {
    background-position: -91px 0;
}

.icon16-config-dir {
    background-position: -111px 0;
}

.icon16-config-config {
    background-position: -129px 0;
}

.icon16-config-db {
    background-position: -147px 0;
}

.icon16-config-cache {
    background-position: -166px 0;
}

.icon16-config-log {
    background-position: -182px 0;
}

.icon16-config-arguments {
    background-position: -204px 0;
}

.icon16-config-github {
    background-position: -224px 0;
}

.icon16-config-mirc {
    background-position: -243px 0;
}

.icon16-config-web {
    background-position: -262px 0;
}

.icon16-config-os {
    background-position: -280px 0;
}

.icon16-config-wiki {
    background-position: -299px 0;
}

[class^="add-client-icon-"], [class*=" add-client-icon-"] {
    background: url("../images/32px_sprite_colored_clients.png");
    height: 32px;
    width: 32px;
    display: inline-block;
    position: relative;
    top: 2px;
    float: left;
}

.add-client-icon-sabnzbd {
    background-position: 0 0;
}

.add-client-icon-nzbget {
    background-position: -34px 0;
}

.add-client-icon-blackhole {
    background-position: -71px 0;
}

.add-client-icon-deluge {
    background-position: -106px 0;
}

.add-client-icon-deluged {
    background-position: -106px 0;
}

.add-client-icon-qbittorrent, .add-client-icon-new-qbittorrent {
    background-position: -138px 0;
}

.add-client-icon-rtorrent {
    background-position: -172px 0;
}

.add-client-icon-download-station {
    background-position: -205px 0;
}

.add-client-icon-transmission {
    background-position: -241px 0;
}

.add-client-icon-utorrent {
    background-position: -273px 0;
}

.add-client-icon-spotnet {
    background-position: -311px 0;
}

.add-client-icon-mlnet {
    background-position: -344px 0;
}

.add-client-icon-rss {
    background-position: -380px 0;
}

.add-client-icon-folder {
    background-position: -416px 0;
}

.add-client-icon-ftp {
    background-position: -452px 0;
}

.add-client-icon-irc {
    background-position: -488px 0;
}

.add-client-icon-putio {
    background-position: -517px 0;
}

/* =======================================================================
config_postProcessing.mako
========================================================================== */

#config div.example {
    padding: 10px;
    background-color: #EFEFEF;
}

.Key {
    width: 100%;
    padding: 6px;
    font-size: 13px;
    background-color: #F4F4F4;
    border: 1px solid #CCCCCC;
    border-collapse: collapse;
    border-spacing: 0;
}

.Key th, .tableHeader {
    padding: 3px 9px;
    margin: 0;
    color: #FFFFFF;
    text-align: center;
    background: none repeat scroll 0 0 #666666;
}

.Key td {
    padding: 1px 5px !important;
}

.Key tr {
    border-bottom: 1px solid #CCCCCC;
}

.Key tr.even {
    background-color: #DFDEDE;
}

.legend {
    position: relative;
    top: 2px;
}

/* =======================================================================
config_notifications.mako
========================================================================== */

div.metadata_options {
    padding: 7px;
    overflow: auto;
    background: #F5F1E4;
    border: 1px solid #CCCCCC;
}

div.metadata_options label:hover {
    color: #FFFFFF;
    background-color: #57442B;
    cursor: pointer;
}

div.metadata_options label {
    display: block;
    padding-left: 7px;
    line-height: 20px;
    color: #003366;
}

div.metadata_example {
    padding: 8px;
}

div.metadata_example label {
    display: block;
    line-height: 21px;
    color: #000000;
    cursor: pointer;
}

div.metadataDiv .disabled {
    color: #CCCCCC;
}

.notifier-icon {
    float: left;
    margin: 6px 4px 0 0;
}

.warning {
    border-color: #F89406;
    background: url("../images/warning16.png") no-repeat right 5px center #FFFFFF;
}

[class^="icon-notifiers-"], [class*=" icon-notifiers-"] {
    background: url("../images/32px_sprite_colored_notifiers.png");
    height: 32px;
    width: 32px;
    display: inline-block;
    position: relative;
    right: 5px;
    top: 2px;
    float: left;
}

.icon-notifiers-slack {
    background: url("../images/Slack_Mark_Web.png");
    background-size: 32px;
}

.icon-notifiers-matrix {
    background: url("../images/Matrix_Web.png");
    background-size: 32px;
    background-color: white;
}

.icon-notifiers-rocketchat {
    background: url("../images/RocketChat.png");
    background-size: 32px;
    background-color: white;
}

.icon-notifiers-discord {
    background: url("../images/32px_discord_notifier.png");
    background-size: 32px;
}

.icon-notifiers-kodi {
    background-position: 0 0;
}

.icon-notifiers-plex {
    background-position: -35px 0;
}

.icon-notifiers-plexth {
    background-position: -69px 0;
}

.icon-notifiers-emby {
    background-position: -104px 0;
}

.icon-notifiers-nmj {
    background-position: -136px 0;
}

.icon-notifiers-syno1 {
    background-position: -168px 0;
}

.icon-notifiers-syno2 {
    background-position: -202px 0;
}

.icon-notifiers-pytivo {
    background-position: -238px 0;
}

.icon-notifiers-growl {
    background-position: -272px 0;
}

.icon-notifiers-prowl {
    background-position: -310px 0;
}

.icon-notifiers-libnotify {
    background-position: -345px 0;
}

.icon-notifiers-pushover {
    background-position: -377px 0;
}

.icon-notifiers-boxcar2 {
    background-position: -414px 0;
}

.icon-notifiers-pushalot {
    background-position: -487px 0;
}

.icon-notifiers-pushbullet {
    background-position: -519px 0;
}

.icon-notifiers-freemobile {
    background-position: -551px 0;
}

.icon-notifiers-telegram {
    background-position: -588px 0;
}

.icon-notifiers-join {
    background-position: -802px 0;
}

.icon-notifiers-twitter {
    background-position: -624px 0;
}

.icon-notifiers-twilio {
    background-position: -836px 0;
}

.icon-notifiers-trakt {
    background-position: -660px 0;
}

.icon-notifiers-email {
    background-position: -695px 0;
}

.icon-notifiers-anime {
    background-position: -733px 0;
}

.icon-notifiers-look {
    background-position: -769px 0;
}

/* =======================================================================
manage*.mako
========================================================================== */

.manageTable th {
    white-space: normal;
    line-height: 24px;
}

.manageTable td.tableright {
    text-align: left;
}

td.tableright {
    text-align: center;
}

.optionWrapper {
    width: 450px;
    margin-left: auto;
    margin-right: auto;
    padding: 6px 12px;
}

.optionWrapper span.selectTitle {
    float: left;
    text-align: left;
    vertical-align: middle;
    width: 225px;
    padding: 6px 0;
}

.optionWrapper div.selectChoices {
    float: left;
    width: 175px;
    margin-left: 25px;
}

.optionWrapper br {
    clear: both;
}

.manageCustom {
    text-align: center;
    padding: 6px;
    margin-left: 25px;
}

.separator {
    font-size: 90%;
    color: #333333;
}

a.whitelink {
    color: #FFFFFF;
}

#failedTable tbody span {
    word-break: break-all;
}

/* =======================================================================
Global
========================================================================== */

.header {
    margin-top: 0;
}

span.path {
    padding: 3px 6px;
    color: #8B0000;
    background-color: #F5F1E4;
}

span.config-path-title {
    padding: 3px 0;
    display: inline-block;
}

.align-left {
    text-align: left;
}

.h2footer {
    margin: -45px 0 8px;
    line-height: 18px;
}

span.quality, span.status {
    font: 12px/13px "Open Sans", verdana, sans-serif;
    background-image: -webkit-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 50%, rgba(0, 0, 0, 0.25));
    background-image: -moz-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 50%, rgba(0, 0, 0, 0.25));
    background-image: -o-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 50%, rgba(0, 0, 0, 0.25));
    background-image: linear-gradient(to bottom, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 50%, rgba(0, 0, 0, 0.25));
    -webkit-box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
    box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
    text-shadow: 0 1px rgba(0, 0, 0, 0.8);
    color: #FFFFFF;
    display: inline-block;
    padding: 2px 4px;
    text-align: center;
    vertical-align: baseline;
    -webkit-border-radius: 4px;
    -moz-border-radius: 4px;
    border-radius: 4px;
    white-space: nowrap;
}

span.any-hd {
    background: #2672B6 repeating-linear-gradient(
            -45deg,
            #2672B6,
            #2672B6 10px,
            #5B990D 10px,
            #5B990D 20px
    );
}

span.Custom {
    background-color: #621993;
}

span.HD {
    background-color: #2672B6;
}

span.HDTV {
    background-color: #2672B6;
}

span.HD720p {
    background-color: #5B990D;
}

span.HD1080p {
    background-color: #2672B6;
}

span.UHD-4K {
    background-color: #7500FF;
}

span.UHD-8K {
    background-color: #410077;
}

span.RawHD {
    background-color: #CD7300;
}

span.RawHDTV {
    background-color: #CD7300;
}

span.SD {
    background-color: #BE2625;
}

span.SDTV {
    background-color: #BE2625;
}

span.SDDVD {
    background-color: #BE2625;
}

span.Any {
    background-color: #666666;
}

span.Unknown {
    background-color: #999999;
}

span.Proper, [class^="pill-snatched"], [class*=" pill-snatched"] {
    background-color: green;
}

span.pill-downloaded {
    background-color: blue;
}

span.pill-wanted {
    background-color: #BE2625;
}

span.pill-failed {
    background-color: red;
}

span.pill-archived {
    background-color: dimgrey;
}

span.false {
    color: #993333;
    /* red */
}

span.true {
    color: #669966;
    /* green */
}

td.false {
    background-color: #993333;
    /* red */
}

td.true {
    background-color: #669966;
    /* green */
}

option.flag {
    padding-left: 35px;
    background: no-repeat 10px 50%;
}

div.fansub-picker h4 {
    margin: 0 auto;
}

div.fansub-picker select {
    width: 100% !important;
}

/* =======================================================================
bootstrap Overrides
========================================================================== */

body {
    padding-top: 60px;
    overflow-y: scroll;
    font-family: roboto, "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
    color: #000000;
}

html * {
    outline: 0 !important;
}

input[type="checkbox"] {
    margin: 2px 0 0;
    line-height: normal;
}

input[type="radio"] {
    margin: 2px 0 0;
    line-height: normal;
}

input, textarea, select, .uneditable-input {
    width: auto;
    color: #000000;
}

pre {
    color: #000000;
    background-color: #F5F5F5;
    border-color: #CCCCCC;
    white-space: pre-line;
}

.navbar-brand {
    padding: 0;
}

.floating-badge {
    position: absolute;
    top: -5px;
    right: -8px;
    padding: 0px 4px;
    background-color: #777777;
    border: 2px solid #959595;
    border-radius: 100px;
    font-size: 12px;
    font-weight: bold;
    text-decoration: none;
    color: white;
}

/* navbar styling */
.navbar-default {
    background-color: #333333;
    filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#555555', endColorstr='#333333');
    background: -webkit-gradient(linear, left top, left bottom, from(#555555), to(#333333));
    background: -moz-linear-gradient(top, #555555, #333333);
    border-color: #3E3F3A;
}

.navbar-default .navbar-collapse,
.navbar-default .navbar-form {
    border-color: #3E3F3A;
}

.navbar-default .navbar-nav > li > a:hover,
.navbar-default .navbar-nav > li > a:focus {
    background-color: #333333;
}

.dropdown-menu > li > a:hover, .dropdown-menu > li > a:focus {
    color: #FFFFFF;
    text-decoration: none;
    background-color: #333333;
}

.navbar-default .navbar-brand {
    color: #FFFFFF;
}

.navbar-default .navbar-brand:hover,
.navbar-default .navbar-brand:focus {
    color: #FFFFFF;
    background-color: transparent;
}

.navbar-default .navbar-text {
    color: #DDDDDD;
}

.navbar-default .navbar-nav > li > a {
    color: #DDDDDD;
}

.navbar-default .navbar-nav > li > a:hover,
.navbar-default .navbar-nav > li > a:focus {
    color: #FFFFFF;
}

.navbar-default .navbar-nav > .active > a,
.navbar-default .navbar-nav > .active > a:hover,
.navbar-default .navbar-nav > .active > a:focus {
    color: #FFFFFF;
    background-color: #333333;
}

.navbar-default .navbar-nav > .disabled > a,
.navbar-default .navbar-nav > .disabled > a:hover,
.navbar-default .navbar-nav > .disabled > a:focus {
    color: #CCCCCC;
    background-color: transparent;
}

.navbar-default .navbar-toggle {
    border-color: #CCCCCC;
}

.navbar-default .navbar-toggle:hover,
.navbar-default .navbar-toggle:focus {
    background-color: #333333;
}

.navbar-default .navbar-toggle .icon-bar {
    background-color: #333333;
}

.navbar-default .navbar-toggle:hover .icon-bar,
.navbar-default .navbar-toggle:focus .icon-bar {
    background-color: #15528F;
}

.navbar-default .navbar-nav > .open > a,
.navbar-default .navbar-nav > .open > a:hover,
.navbar-default .navbar-nav > .open > a:focus {
    background-color: #333333;
    color: #FFFFFF;
}

.navbar-default .navbar-nav > li.navbar-split > a {
    display: inline-block;
    float: left;
}

.navbar-default .navbar-nav > li.navbar-split > a:first-child {
    padding-right: 8px;
}

.navbar-default .navbar-nav > li.navbar-split > a:nth-child(2) {
    padding-left: 8px;
}

@media (max-width: 767px) {
    .navbar-default .navbar-nav .open .dropdown-menu > li > a {
        color: #DDDDDD;
    }

    .navbar-default .navbar-nav .open .dropdown-menu > li > a:hover,
    .navbar-default .navbar-nav .open .dropdown-menu > li > a:focus {
        color: #FFFFFF;
        background-color: transparent;
    }

    .navbar-default .navbar-nav .open .dropdown-menu > .active > a,
    .navbar-default .navbar-nav .open .dropdown-menu > .active > a:hover,
    .navbar-default .navbar-nav .open .dropdown-menu > .active > a:focus {
        color: #FFFFFF;
        background-color: #333333;
    }

    .navbar-default .navbar-nav .open .dropdown-menu > .disabled > a,
    .navbar-default .navbar-nav .open .dropdown-menu > .disabled > a:hover,
    .navbar-default .navbar-nav .open .dropdown-menu > .disabled > a:focus {
        color: #CCCCCC;
        background-color: transparent;
    }
}

.navbar-default .navbar-link {
    color: #DDDDDD;
}

.navbar-default .navbar-link:hover {
    color: #FFFFFF;
}

.navbar-default .btn-link {
    color: #98978B;
}

.navbar-default .btn-link:hover,
.navbar-default .btn-link:focus {
    color: #DDDDDD;
}

.navbar-default .btn-link[disabled]:hover,
fieldset[disabled] .navbar-default .btn-link:hover,
.navbar-default .btn-link[disabled]:focus,
fieldset[disabled] .navbar-default .btn-link:focus {
    color: #CCCCCC;
}

.dropdown-menu > li > a:hover, .dropdown-menu > li > a:focus {
    color: #262626;
    text-decoration: none;
    background-color: #F5F5F5;
}

.dropdown-menu > li > a {
    padding: 4px 36px 4px 20px;
}

.dropdown-menu {
    background-color: #F5F1E4;
    border: 1px solid rgba(0, 0, 0, 0.15);
    box-shadow: 0 6px 12px rgba(0, 0, 0, 0.176);
}

.form-control {
    color: #000000;
}

.form-control-inline {
    min-width: 0;
    width: auto;
    display: inline;
}

.btn {
    display: inline-block;
    *display: inline;
    padding: 4px 10px 4px;
    margin-bottom: 0;
    *margin-left: .3em;
    font-size: 12px;
    line-height: 16px;
    *line-height: 20px;
    color: #333333;
    text-align: center;
    text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
    vertical-align: middle;
    cursor: pointer;
    background-color: #F5F5F5;
    *background-color: #E6E6E6;
    background-image: -ms-linear-gradient(top, #FFFFFF, #E6E6E6);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#FFFFFF), to(#E6E6E6));
    background-image: -webkit-linear-gradient(top, #FFFFFF, #E6E6E6);
    background-image: -o-linear-gradient(top, #FFFFFF, #E6E6E6);
    background-image: linear-gradient(top, #FFFFFF, #E6E6E6);
    background-image: -moz-linear-gradient(top, #FFFFFF, #E6E6E6);
    background-repeat: repeat-x;
    border: 1px solid #CCCCCC;
    *border: 0;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    border-color: #E6E6E6 #E6E6E6 #BFBFBF;
    border-bottom-color: #B3B3B3;
    -webkit-border-radius: 4px;
    -moz-border-radius: 4px;
    border-radius: 4px;
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
    filter: progid:dximagetransform.microsoft.gradient(enabled=false);
    *zoom: 1;
    -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
    -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
    box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
}

.modal-header {
    padding: 9px 15px;
    border-bottom: 1px solid #EEEEEE;
    background-color: #F5F1E4;
    -webkit-border-top-left-radius: 5px;
    -webkit-border-top-right-radius: 5px;
    -moz-border-radius-topleft: 5px;
    -moz-border-radius-topright: 5px;
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
}

.btn:hover,
.btn:active,
.btn.active,
.btn.disabled,
.btn[disabled] {
    background-color: #E6E6E6;
    *background-color: #D9D9D9;
}

.btn:active,
.btn.active {
    background-color: #CCCCCC \9;
}

.btn:first-child {
    *margin-left: 0;
}

.btn:hover {
    color: #333333;
    text-decoration: none;
    *background-color: #D9D9D9;
    background: #E6E6E6 0 -15px;
    -webkit-transition: background-position 0.1s linear;
    -moz-transition: background-position 0.1s linear;
    -ms-transition: background-position 0.1s linear;
    -o-transition: background-position 0.1s linear;
    transition: background-position 0.1s linear;
}

.btn:focus {
    outline: thin dotted #333333;
    outline: 5px auto -webkit-focus-ring-color;
    outline-offset: -2px;
}

.btn.active,
.btn:active {
    background-color: #D9D9D9 \9;
    background: #E6E6E6 none;
    outline: 0;
    -webkit-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
    -moz-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
    box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
}

.btn.disabled,
.btn[disabled] {
    cursor: default;
    background: #E6E6E6 none;
    opacity: 0.65;
    filter: alpha(opacity=65);
    -webkit-box-shadow: none;
    -moz-box-shadow: none;
    box-shadow: none;
}

.btn-large {
    padding: 9px 14px;
    font-size: 15px;
    line-height: normal;
    -webkit-border-radius: 5px;
    -moz-border-radius: 5px;
    border-radius: 5px;
}

.btn-large [class^="icon-"] {
    margin-top: 1px;
}

.btn-small {
    padding: 5px 9px;
    font-size: 11px;
    line-height: 16px;
}

.btn-small [class^="icon-"] {
    margin-top: -1px;
}

.btn-mini {
    padding: 2px 6px;
    font-size: 11px;
    line-height: 14px;
}

.btn-primary,
.btn-primary:hover,
.btn-warning,
.btn-warning:hover,
.btn-danger,
.btn-danger:hover,
.btn-success,
.btn-success:hover,
.btn-info,
.btn-info:hover,
.btn-inverse,
.btn-inverse:hover {
    color: #FFFFFF;
    text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
}

.btn-primary.active,
.btn-warning.active,
.btn-danger.active,
.btn-success.active,
.btn-info.active,
.btn-inverse.active {
    color: rgba(255, 255, 255, 0.75);
}

.btn-primary {
    background-color: #0074CC;
    *background-color: #0055CC;
    background-image: -ms-linear-gradient(top, #0088CC, #0055CC);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#0088CC), to(#0055CC));
    background-image: -webkit-linear-gradient(top, #0088CC, #0055CC);
    background-image: -o-linear-gradient(top, #0088CC, #0055CC);
    background-image: -moz-linear-gradient(top, #0088CC, #0055CC);
    background-image: linear-gradient(top, #0088CC, #0055CC);
    background-repeat: repeat-x;
    border-color: #0055CC #0055CC #003580;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#0088cc', endColorstr='#0055cc', GradientType=0);
    filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}

.btn-primary:hover,
.btn-primary:active,
.btn-primary.active,
.btn-primary.disabled,
.btn-primary[disabled] {
    background-color: #0055CC;
    *background-color: #004AB3;
}

.btn-primary:active,
.btn-primary.active {
    background-color: #004099 \9;
}

.btn-warning {
    background-color: #FAA732;
    *background-color: #F89406;
    background-image: -ms-linear-gradient(top, #FBB450, #F89406);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#FBB450), to(#F89406));
    background-image: -webkit-linear-gradient(top, #FBB450, #F89406);
    background-image: -o-linear-gradient(top, #FBB450, #F89406);
    background-image: -moz-linear-gradient(top, #FBB450, #F89406);
    background-image: linear-gradient(top, #FBB450, #F89406);
    background-repeat: repeat-x;
    border-color: #F89406 #F89406 #AD6704;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#fbb450', endColorstr='#f89406', GradientType=0);
    filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}

.btn-warning:hover,
.btn-warning:active,
.btn-warning.active,
.btn-warning.disabled,
.btn-warning[disabled] {
    background-color: #F89406;
    *background-color: #DF8505;
}

.btn-warning:active,
.btn-warning.active {
    background-color: #C67605 \9;
}

.btn-danger {
    background-color: #DA4F49;
    *background-color: #BD362F;
    background-image: -ms-linear-gradient(top, #EE5F5B, #BD362F);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#EE5F5B), to(#BD362F));
    background-image: -webkit-linear-gradient(top, #EE5F5B, #BD362F);
    background-image: -o-linear-gradient(top, #EE5F5B, #BD362F);
    background-image: -moz-linear-gradient(top, #EE5F5B, #BD362F);
    background-image: linear-gradient(top, #EE5F5B, #BD362F);
    background-repeat: repeat-x;
    border-color: #BD362F #BD362F #802420;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#bd362f', GradientType=0);
    filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}

.btn-danger:hover,
.btn-danger:active,
.btn-danger.active,
.btn-danger.disabled,
.btn-danger[disabled] {
    background-color: #BD362F;
    *background-color: #A9302A;
}

.btn-danger:active,
.btn-danger.active {
    background-color: #942A25 \9;
}

.btn-success {
    background-color: #5BB75B;
    *background-color: #51A351;
    background-image: -ms-linear-gradient(top, #62C462, #51A351);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#62C462), to(#51A351));
    background-image: -webkit-linear-gradient(top, #62C462, #51A351);
    background-image: -o-linear-gradient(top, #62C462, #51A351);
    background-image: -moz-linear-gradient(top, #62C462, #51A351);
    background-image: linear-gradient(top, #62C462, #51A351);
    background-repeat: repeat-x;
    border-color: #51A351 #51A351 #387038;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#62c462', endColorstr='#51a351', GradientType=0);
    filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}

.btn-success:hover,
.btn-success:active,
.btn-success.active,
.btn-success.disabled,
.btn-success[disabled] {
    background-color: #51A351;
    *background-color: #499249;
}

.btn-success:active,
.btn-success.active {
    background-color: #408140 \9;
}

.btn-info {
    background-color: #49AFCD;
    *background-color: #2F96B4;
    background-image: -ms-linear-gradient(top, #5BC0DE, #2F96B4);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#5BC0DE), to(#2F96B4));
    background-image: -webkit-linear-gradient(top, #5BC0DE, #2F96B4);
    background-image: -o-linear-gradient(top, #5BC0DE, #2F96B4);
    background-image: -moz-linear-gradient(top, #5BC0DE, #2F96B4);
    background-image: linear-gradient(top, #5BC0DE, #2F96B4);
    background-repeat: repeat-x;
    border-color: #2F96B4 #2F96B4 #1F6377;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#5bc0de', endColorstr='#2f96b4', GradientType=0);
    filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}

.btn-info:hover,
.btn-info:active,
.btn-info.active,
.btn-info.disabled,
.btn-info[disabled] {
    background-color: #2F96B4;
    *background-color: #2A85A0;
}

.btn-info:active,
.btn-info.active {
    background-color: #24748C \9;
}

.btn-inverse {
    background-color: #414141;
    *background-color: #222222;
    background-image: -ms-linear-gradient(top, #555555, #222222);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#555555), to(#222222));
    background-image: -webkit-linear-gradient(top, #555555, #222222);
    background-image: -o-linear-gradient(top, #555555, #222222);
    background-image: -moz-linear-gradient(top, #555555, #222222);
    background-image: linear-gradient(top, #555555, #222222);
    background-repeat: repeat-x;
    border-color: #222222 #222222 #000000;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#555555', endColorstr='#222222', GradientType=0);
    filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}

.btn-inverse:hover,
.btn-inverse:active,
.btn-inverse.active,
.btn-inverse.disabled,
.btn-inverse[disabled] {
    background-color: #222222;
    *background-color: #151515;
}

.btn-inverse:active,
.btn-inverse.active {
    background-color: #080808 \9;
}

.btn-xs {
    padding: 1px 5px;
    font-size: 12px;
    line-height: 1.5;
    border-radius: 3px;
}

@media (min-width: 768px) {
    .navbar .nav > li > .dropdown-menu:after {
        position: absolute;
        top: -6px;
        right: 12px;
        display: inline-block;
        border-right: 6px solid transparent;
        border-bottom: 6px solid #F5F1E4;
        border-left: 6px solid transparent;
        content: "";
    }
}

label {
    font-weight: normal;
}

pre {
    border: 1px solid;
}

.alert {
    padding: 10px;
    text-align: center;
}

/* =======================================================================
input sizing (for config pages)
========================================================================== */

#config select {
    min-width: 0;
    display: inline;
}

.btn-inline {
    margin-top: -3px;
}

.input75 {
    max-width: 75px;
    margin-top: -4px;
}

.input100 {
    max-width: 100px;
    margin-top: -4px;
}

.input150 {
    max-width: 150px;
    margin-top: -4px;
}

.input200 {
    max-width: 200px;
    margin-top: -4px;
}

.input250 {
    max-width: 250px;
    margin-top: -4px;
}

.input300 {
    max-width: 300px;
    margin-top: -4px;
}

.input350 {
    max-width: 350px;
    margin-top: -4px;
}

/* =======================================================================
formWizard.css
========================================================================== */

fieldset.sectionwrap {
    width: 800px;
    padding: 5px;
    text-align: left;
    border-width: 0;
}

legend.legendStep {
    color: #57442B;
    margin-bottom: 0;
}

div.stepsguide {
    margin-bottom: 15px;
    overflow: hidden;
    text-align: left;
    cursor: pointer;
}

div.stepsguide .step {
    float: left;
    width: 33%;
    font: bold 24px Arial;
}

div.stepsguide .step p {
    margin: 12px 0;
    border-bottom: 4px solid #57442B;
}

div.stepsguide .disabledstep {
    color: #C4C4C4;
}

div.stepsguide .disabledstep p {
    border-bottom: 4px solid #8A775E;
}

div.stepsguide .step .smalltext {
    font-size: 13px;
    font-weight: normal;
}

div.formpaginate {
    padding: 0 20px;
    margin-top: 1em;
    overflow: auto;
    font-weight: bold;
    text-align: center;
}

div.formpaginate .prev, div.formpaginate .next {
    padding: 3px 6px;
    color: #FFFFFF;
    cursor: hand;
    cursor: pointer;
    background: #57442B;
    -webkit-border-radius: 6px;
    -moz-border-radius: 6px;
    border-radius: 6px;
}

.stepDiv {
    padding: 15px 0;
}

#tabs .nocheck,
.stepDiv .nocheck {
    padding-left: 16px;
}

#tabs label span.component-title,
.stepDiv label span.component-title {
    padding-bottom: 10px;
    float: left;
    width: 172px;
    margin-right: 10px;
    font-size: 13px;
    font-weight: bold;
}

#tabs label span.component-desc,
.stepDiv label span.component-desc {
    padding-bottom: 10px;
    font-size: 12px;
    font-weight: normal;
    display: block;
    width: 475px;
    margin-left: 182px;
}

div.field-pair select,
div.field-pair input {
    margin: 6px 6px 6px 0;
    float: left;
}

/* step 3 related */
#customQuality {
    display: block;
    padding: 10px 0;
    overflow: hidden;
    clear: both;
}

#customQualityWrapper div.component-group-desc {
    float: left;
    width: 172px;
    margin-right: 10px;
}

#customQualityWrapper div.component-group-desc p {
    width: 85%;
    margin: .8em 0;
    font-size: 1.13em;
    color: #666666;
}

/* =======================================================================
tablesorter.css
========================================================================== */

.tablesorter {
    width: 100%;
    margin-right: auto;
    margin-left: auto;
    color: #000000;
    text-align: left;
    background-color: #DDDDDD;
    border-spacing: 0;
}

.tablesorter th,
.tablesorter td {
    padding: 4px;
    border-top: #FFFFFF 1px solid;
    border-left: #FFFFFF 1px solid;
    vertical-align: middle;
}

/* add extra border at right edge */
.tablesorter th:last-child,
.tablesorter td:last-child {
    border-right: #FFFFFF 1px solid;
}

.tablesorter th {
    color: #FFFFFF;
    text-align: center;
    text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.3);
    background-color: #333333;
    border-collapse: collapse;
    font-weight: normal;
}

.tablesorter .tablesorter-header {
    padding: 4px 18px 4px 18px;
    cursor: pointer;
    background: inherit url(data:image/gif;base64,R0lGODlhFQAJAIAAAP///////yH5BAEAAAEALAAAAAAVAAkAAAIXjI+AywnaYnhUMoqt3gZXPmVg94yJVQAAOw==) no-repeat center right;
}

.tablesorter thead .tablesorter-headerDesc {
    background: #555555 url(data:image/gif;base64,R0lGODlhFQAEAIAAAP///////yH5BAEAAAEALAAAAAAVAAQAAAINjB+gC+jP2ptn0WskLQA7) no-repeat center right;
}

.tablesorter thead .tablesorter-headerAsc {
    background: #555555 url(data:image/gif;base64,R0lGODlhFQAEAIAAAP///////yH5BAEAAAEALAAAAAAVAAQAAAINjI8Bya2wnINUMopZAQA7) no-repeat center right;
}

.tablesorter thead .sorter-false {
    background-image: none;
    padding: 4px;
    cursor: default;
}

thead.tablesorter-stickyHeader {
    border-top: 2px solid #FFFFFF;
    border-bottom: 2px solid #FFFFFF;
}

/* Zebra Widget - row alternating colors */
.tablesorter tr.odd, .sickchillTable tr.odd {
    background-color: #F5F1E4;
}

.tablesorter tr.even, .sickchillTable tr.even {
    background-color: #DFDACF;
}

/* filter widget */
.tablesorter .filtered {
    display: none;
}

.tablesorter input.tablesorter-filter {
    width: 98%;
    height: auto;
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
}

.tablesorter tr.tablesorter-filter-row,
.tablesorter tr.tablesorter-filter-row td {
    text-align: center;
    background: #EEEEEE;
    border-bottom: 1px solid #DDDDDD;
}

/* add extra border at left edge */
.tablesorter tr.tablesorter-filter-row td:first-child {
    border-left: #FFFFFF 1px solid;
}

/* add extra border at right edge */
.tablesorter tr.tablesorter-filter-row td:last-child {
    border-right: #FFFFFF 1px solid;
}

/* hidden filter row */
.tablesorter-filter-row.hideme td {
    /*** *********************************************** ***/
    /*** change this padding to modify the thickness     ***/
    /*** of the closed filter row (height = padding x 2) ***/
    padding: 2px;
    /*** *********************************************** ***/
    margin: 0;
    line-height: 0;
    cursor: pointer;
}

.tablesorter-filter-row.hideme * {
    height: 1px;
    min-height: 0;
    border: 0;
    padding: 0;
    margin: 0;
    /* don't use visibility: hidden because it disables tabbing */
    opacity: 0;
    filter: alpha(opacity=0);
}

/* optional disabled input styling */
.tablesorter input.tablesorter-filter-row .disabled {
    display: none;
}

.tablesorter-header-inner {
    padding: 0 2px;
    text-align: center;
}

.tablesorter tfoot tr {
    color: #FFFFFF;
    text-align: center;
    text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.3);
    background-color: #333333;
    border-collapse: collapse;
}

.tablesorter tfoot a {
    color: #FFFFFF;
    text-decoration: none;
}

/* =======================================================================
token-input.css
========================================================================== */

ul.token-input-list {
    overflow: hidden;
    height: auto !important;
    height: 1%;
    width: 273px;
    border: 1px solid #CCCCCC;
    cursor: text;
    font-size: 10px;
    z-index: 999;
    margin: 0;
    padding: 0 0 1px 0;
    background-color: #FFFFFF;
    list-style-type: none;
    border-radius: 3px;
}

ul.token-input-list li {
    list-style-type: none;
}

ul.token-input-list li input {
    border: 0;
    padding: 3px 4px;
    background-color: white;
}

li.token-input-token {
    overflow: hidden;
    height: auto !important;
    height: 1%;
    margin: 3px;
    padding: 3px 5px 0 5px;
    background-color: #D0EFA0;
    color: #000000;
    font-weight: bold;
    cursor: default;
    display: block;
}

li.token-input-token img {
    padding-right: 4px;
    float: left;
}

li.token-input-token input {
    padding-top: 2px !important;
    padding-right: 4px !important;
    float: left;
}

li.token-input-token p {
    float: left;
    padding: 0;
    margin: 0;
    line-height: 2.0 !important;
}

li.token-input-token span {
    float: right;
    color: #777777;
    cursor: pointer;
}

li.token-input-selected-token {
    background-color: #08844E;
    color: #FFFFFF;
}

li.token-input-selected-token span {
    color: #BBBBBB;
}

li.token-input-input-token input {
    margin: 3px 3px 3px 3px !important;
}

div.token-input-dropdown {
    position: absolute;
    width: 273px;
    overflow: hidden;
    border-left: 1px solid #CCCCCC;
    border-right: 1px solid #CCCCCC;
    border-bottom: 1px solid #CCCCCC;
    cursor: default;
    font-size: 11px;
    z-index: 1;
    background-color: #FFFFFF;
    color: #000000;
}

div.token-input-dropdown p {
    margin: 0;
    padding: 3px;
    font-weight: bold;
    color: #777777;
}

div.token-input-dropdown ul {
    margin: 0;
    padding: 0;
}

div.token-input-dropdown ul li {
    background-color: #FFFFFF;
    padding: 3px;
    list-style-type: none;
}

div.token-input-dropdown ul li.token-input-dropdown-item {
    background-color: #FAFAFA;
}

div.token-input-dropdown ul li.token-input-dropdown-item2 {
    background-color: #FFFFFF;
}

div.token-input-dropdown ul li em {
    font-weight: bold;
    font-style: normal;
}

div.token-input-dropdown ul li.token-input-selected-dropdown-item {
    background-color: #6196C2;
}

span.token-input-delete-token {
    margin: 0 1px;
}

.red-text {
    color: #DD3333
}

.clear-left {
    clear: left
}

.nextline-block {
    display: block
}

/* =======================================================================
jquery.confirm.css
========================================================================== */

#confirmOverlay {
    width: 100%;
    height: 100%;
    position: fixed;
    top: 0;
    left: 0;
    background: url('../images/bg.gif');
    background: -moz-linear-gradient(rgba(0, 0, 0, 0.5), rgba(0, 0, 0, 0.5)) repeat-x rgba(0, 0, 0, 0.5);
    background: -webkit-gradient(linear, 0% 0%, 0% 100%, from(rgba(0, 0, 0, 0.5)), to(rgba(0, 0, 0, 0.5))) repeat-x rgba(0, 0, 0, 0.5);
    z-index: 100000;
}

#confirmBox {
    background: #F5F1E4;
    width: 460px;
    position: fixed;
    left: 50%;
    top: 50%;
    margin: -130px 0 0 -230px;
    border: 1px solid #111111;
    box-shadow: 0 0 12px 0 rgba(0, 0, 0, 0.175);
}

#confirmBox h1,
#confirmBox p {
    padding: 6px 10px;
}

#confirmBox h1 {
    background-color: #333333;
    border-bottom: 1px solid #111111;
    color: #FFFFFF;
    margin: 0;
    font-size: 22px;
    text-shadow: 0 1px 1px rgba(0, 0, 0, 0.75);
}

#confirmBox p {
    padding-top: 20px;
    color: #000000;
    text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
}

#confirmButtons {
    padding: 15px 0;
    text-align: center;
}

#confirmBox .button {
    margin-right: 15px;
    padding: 2px 20px;
    text-decoration: none;
    display: inline-block;
    color: #FFFFFF;
    text-align: center;
    text-shadow: 0 1px 1px rgba(0, 0, 0, 0.75);
    background-clip: padding-box;
    border: 1px solid #111111;
    border-radius: 3px;
    cursor: pointer;
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
    background-image: -webkit-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    background-image: -moz-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    background-image: -o-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    background-image: linear-gradient(to bottom, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    -webkit-box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
    box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
}

#confirmBox .button:last-child {
    margin-right: 0;
}

#confirmBox .green {
    background-color: #3F7636;
}

#confirmBox .green:hover {
    background-color: #48873E;
}

#confirmBox .red {
    background-color: #8D2D2B;
}

#confirmBox .red:hover {
    background-color: #A13331;
}

/* =======================================================================
pnotify.css
========================================================================== */

.ui-pnotify-sticker {
    margin-top: -12px;
}

.ui-pnotify-closer {
    margin-top: -12px;
    margin-right: -10px;
}

/* =======================================================================
login.css
========================================================================== */

.login h1 {
    font-size: 60px;
}

/* =======================================================================
IMDB Popular
========================================================================== */

.popularShow {
    margin-bottom: 30px;
}

.popularShow h3 {
    padding: 0;
    display: inline-block;
    margin: 0 30px 0 0;
}

.popularShow .left {
    float: left;
    width: 100px;
}

.popularShow .right {
    float: left;
    width: 600px;
}

.popularShow .year {
    font-style: italic;
    opacity: 0.7;
}

.popularShow .coverImage {
    width: 80%;
    padding-left: 20px;
    margin-top: 4px;
}

.popularShow .rating {
    font-size: 90%;
    display: inline-block;
    margin-left: 0;
}

.popularShow p {
    margin-bottom: 0;
}

input.tablesorter-filter.disabled {
    display: none;
}

/* This fixes bold fon be used */
/* @TODO replace this with proper CSS overrides */
.ui-state-default, .ui-widget-content .ui-state-default, .ui-widget-header {
    font-weight: normal !important;
}

@media (max-width: 767px) {
    .pull-xs-left {
        float: left;
    }

    .pull-xs-right {
        float: right;
    }

    .no-padding-xs {
        padding: 0;
    }

    .nocheck {
        padding-left: 0 !important;
    }

    .tvshowImg, .movieImg {
        margin-bottom: 10px;
    }

    .poster-container {
        display: inline-block;
        width: 100%;
    }

    .poster-container img {
        display: block;
        margin: 0 auto;
    }

    .info-container {
        display: block;
        width: 100%;
    }

    .col-md-12 > .horizontal-scroll {
        margin: 0 -13px;
    }

    .pick-show-group {
        width: 100%;
        margin: 0 auto;
    }
}

@media (min-width: 768px) and (max-width: 991px) {
    .pull-sm-left {
        float: left;
    }

    .pull-sm-right {
        float: right;
    }

    .no-padding-sm {
        padding: 0;
    }

    .tvshowImg, .movieImg {
        max-height: 245px;
    }

    .poster-container {
        float:left;
        display: inline-block;
        width: 100%;
    }

    .poster-container img {
        display: block;
        margin: 0 auto;
    }

    .info-container {
        display: block;
        width: 100%;
    }

    .pick-show-group {
        width: 100%;
        margin: 0 auto;
    }
}

@media (min-width: 992px) and (max-width: 1199px) {
    .pull-md-left {
        float: left;
    }

    .pull-md-right {
        float: right;
    }

    .tvshowImg, .movieImg {
        max-height: 269px;
    }
}

@media (min-width: 1200px) {
    .pull-lg-left {
        float: left;
    }

    .pull-lg-right {
        float: right;
    }

    .tvshowImg, .movieImg {
        max-height: 269px;
    }
}

.horizontal-scroll {
    overflow-x: auto;
}

#episodeDir {
    width: 100%;
}

[class^="displayshow-play-icon-"], [class*=" displayshow-play-icon-"] {
    background: url("../images/16px_sprite_colored_display-show-players.png");
    height: 16px;
    width: 16px;
    display: inline-block;
    position: unset;
    top: 3px;
}

.displayshow-play-icon-kodi {
    background-position: 0 0;
}
