#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2016-9-12

@author: drummersbrother
'''

LOCALE = [
    ["just nu", "ett tag"],
    ["för %s sekunder sedan", "om %s sekunder"],
    ["för 1 minut sedan", "om 1 minut"],
    ["för %s minuter sedan", "om %s minuter"],
    ["för 1 timme sedan", "om 1 timme"],
    ["för %s timmar sedan", "om %s timmar"],
    ["för 1 dag sedan", "om 1 dag"],
    ["för %s dagar sedan", "om %s dagar"],
    ["för 1 vecka sedan", "om 1 vecka"],
    ["för %s veckor sedan", "om %s veckor"],
    ["för 1 månad sedan", "om 1 månad"],
    ["för %s månader sedan", "om %s månader"],
    ["för 1 år sedan", "om 1 år"],
    ["för %s år sedan", "om %s år"],
]
