"""
Refiners enrich a :class:`~subliminal.video.Video` object by adding information to it.

A refiner is a simple function:

.. py:function:: refine(video, **kwargs)

    :param video: the video to refine.
    :type video: :class:`~subliminal.video.Video`
    :param \*\*kwargs: additional parameters for refiners.

"""
