#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-01-26

@author: meyti[mahdi.ghane.g]
'''


LOCALE = [
    ["هم اکنون",
     "به زودی"],
    ["%s ثانیه پیش",
     "%s ثانیه بعد"],
    ["یک دقیقه پیش",
     "یک دقیقه بعد"],
    ["%s دقیقه پیش",
     "%s دقیقه بعد"],
    ["یک ساعت پیش",
     "یک ساعت بعد"],
    ["%s ساعت پیش",
     "%s ساعت بعد"],
    ["دیروز",
     "فردا"],
    ["%s روز پیش",
     "%s روز بعد"],
    ["هفته پیش",
     "هفته بعد"],
    ["%s هفته پیش",
     "%s هفته بعد"],
    ["ماه پیش",
     "ماه بعد"],
    ["%s ماه پیش",
     "%s ماه بعد"],
    ["پارسال",
     "سال بعد"],
    ["%s سال پیش",
     "%s سال بعد"],
]
