#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2016-5-26

@author: HAKASHUN
'''

LOCALE = [
    [u"たった今", u"すぐに"],
    [u"%s秒前", u"%s秒以内"],
    [u"1分前", u"1分以内"],
    [u"%s分前", u"%s分以内"],
    [u"1時間前", u"1時間以内"],
    [u"%s時間前", u"%s時間以内"],
    [u"昨日", u"1日以内"],
    [u"%s日前", u"%s日以内"],
    [u"先週", u"1週間以内"],
    [u"%s週間前", u"%s週間以内"],
    [u"1ヶ月前", u"1ヶ月以内"],
    [u"%sヶ月前", u"%sヶ月以内"],
    [u"1年前", u"1年以内"],
    [u"%s年前", u"%s年以内"],
]