class ShowResult(object):
    def __init__(self, title, first_aired, next_airs, last_aired, status, language, indexer, description):
        self.title = title
        self.first_aired = first_aired
        self.next_airs = next_airs
        self.last_aired = last_aired
        self.status = status
        self.language = language
        self.indexer = indexer
        self.description = description
