#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["éppen most", "éppen most"],
    ["%s másodperce", "%s másodpercen belül"],
    ["1 perce", "1 percen belül"],
    ["%s perce", "%s percen belül"],
    ["1 órája", "1 órán belül"],
    ["%s órája", "%s órán belül"],
    ["1 napja", "1 napon belül"],
    ["%s napja", "%s napon belül"],
    ["1 hete", "1 héten belül"],
    ["%s hete", "%s héten belül"],
    ["1 hónapja", "1 hónapon belül"],
    ["%s hónapja", "%s hónapon belül"],
    ["1 éve", "1 éven belül"],
    ["%s éve", "%s éven belül"]
]
        