/* =======================================================================
inc_top.mako
========================================================================== */

.ui-dialog,
.ui-dialog-buttonpane {
    background: #2A2A2A !important;
}

.ui-widget-content {
    background: #606060;
    border: 1px solid #111111;
    color: #FFFFFF;
}

.ui-widget-content a {
    color: #FFFFFF;
}

.ui-widget-content a:hover {
    color: #09A2FF;
    text-decoration: none;
}

.ui-widget-header {
    background: #3D3D3D;
    border: 1px solid #111111;
    color: #FFFFFF;
}

.ui-state-default,
.ui-widget-content .ui-state-default,
.ui-widget-header .ui-state-default {
    border: 1px solid #111111;
}

.ui-state-hover,
.ui-widget-content .ui-state-hover,
.ui-widget-header .ui-state-hover,
.ui-state-focus,
.ui-widget-content .ui-state-focus,
.ui-widget-header .ui-state-focus {
    background: #3D3D3D;
}

.ui-state-active,
.ui-widget-content .ui-state-active,
.ui-widget-header .ui-state-active {
    background: #3D3D3D;
}

.ui-icon,
.ui-widget-content .ui-icon {
    background-image: url("../css/lib/images/ui-icons_ffffff_256x240.png");
}

.ui-state-default .ui-icon {
    background-image: url('../css/lib/images/ui-icons_09a2ff_256x240.png');
}

.ui-widget-overlay {
    background: #000000 url("../css/lib/images/ui-bg_flat_0_000000_40x100.png") 50% 50% repeat-x;
}

.ui-state-default a, .ui-state-default a:link, .ui-state-default a:visited {
    color: #FFFFFF;
    text-decoration: none;
}

.ui-dialog .ui-dialog-titlebar-close {
    background: #333333;
}

.ui-tabs {
    padding: 0;
    background: none;
    border-width: 0;
}

.ui-tabs .ui-tabs-panel {
    background-color: #3D3D3D !important;
    border: 1px solid #111111 !important;
}

.ui-tabs-nav > :not(.ui-tabs-active) {
    background: #333333;
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
}

/* =======================================================================
inc_bottom.mako
========================================================================== */

.footer {
    color: #FFFFFF;
}

.footerhighlight {
    color: #09A2FF;
}

/* =======================================================================
home.mako
========================================================================== */

.imgbanner .banner {
    border: 1px solid #111111;
}

.imgsmallposter .small {
    border: 1px solid #111111;
}

.progressbarText {
    text-shadow: 0 0 0.1em #000000;
    color: #FFFFFF;
}

.progress-100 {
    background-image: -moz-linear-gradient(#395F07, #2A4705) !important;
    background-image: linear-gradient(#395F07, #2A4705) !important;
    background-image: -webkit-linear-gradient(#395F07, #2A4705) !important;
    background-image: -o-linear-gradient(#395F07, #2A4705) !important;
}

.progress-80 {
    background-image: -moz-linear-gradient(#A6CF41, #5B990D) !important;
    background-image: linear-gradient(#A6CF41, #5B990D) !important;
    background-image: -webkit-linear-gradient(#A6CF41, #5B990D) !important;
    background-image: -o-linear-gradient(#A6CF41, #5B990D) !important;
}

.progress-60 {
    background-image: -moz-linear-gradient(#FAD440, #F2A70D) !important;
    background-image: linear-gradient(#FAD440, #F2A70D) !important;
    background-image: -webkit-linear-gradient(#FAD440, #F2A70D) !important;
    background-image: -o-linear-gradient(#FAD440, #F2A70D) !important;
}

.progress-40 {
    background-image: -moz-linear-gradient(#FAB543, #F2700D) !important;
    background-image: linear-gradient(#FAB543, #F2700D) !important;
    background-image: -webkit-linear-gradient(#FAB543, #F2700D) !important;
    background-image: -o-linear-gradient(#FAB543, #F2700D) !important;
}

.progress-20 {
    background-image: -moz-linear-gradient(#DA5945, #B11A10) !important;
    background-image: linear-gradient(#DA5945, #B11A10) !important;
    background-image: -webkit-linear-gradient(#DA5945, #B11A10) !important;
    background-image: -o-linear-gradient(#DA5945, #B11A10) !important;
}

.loading-spinner {
    background-image: url("../images/loading32-dark.gif");
}

.show-container, .movie-container {
    background-color: #333333;
    border-color: #333333;
}

.show-title:after, .movie-title:after {
    background-image: -webkit-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: -moz-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: -ms-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: -o-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: linear-gradient(to left, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
}

.show-date:after, .movie-date:after{
    background-image: -webkit-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: -moz-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: -ms-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: -o-linear-gradient(right, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
    background-image: linear-gradient(to left, rgba(51, 51, 51, 1), rgba(51, 51, 51, 0));
}

td.tvShow a {
    color: #FFFFFF;
}

td.tvShow a:hover {
    color: #09A2FF;
}

.popover {
    background-color: #333333;
}

.popover-content {
    background-color: #333333;
}

.popover.bottom .arrow:after {
    border-bottom-color: #333333;
}

/* =======================================================================
home_newShow.mako
========================================================================== */

#displayText {
    background-color: #3D3D3D;
    border: 1px solid #111111;
}

.new-show-table {
    color: #FFFFFF !important;
    border-right: 1px solid black;
    border-bottom: 1px solid black;
}

.new-show-table tr.in-list,
.new-show-table tr.in-list a {
    color: #ACACAC !important;
}

/* =======================================================================
home_addExistingShow.mako
========================================================================== */

ul#rootDirStaticList li {
    background: #3D3D3D;
    color: white;
}

/* =======================================================================
home_trendingShows.mako
========================================================================== */

.traktContainer {
    background-color: #333333;
}

/* =======================================================================
displayShow.mako
========================================================================== */

#prevShow,
#nextShow,
#topcontrol {
    -webkit-filter: none;
    filter: none;
}

h1.title {
    border-bottom: 1px solid #555555;
}

ul.tags li {
    background: url(../images/tag.png) no-repeat scroll 5px 4px #15528F;
}

.tvshowImg, .movieImg {
    border: 1px solid #111111;
}

#summary {
    background-color: #3D3D3D;
    border: 1px solid #111111;
}

.displayShowTable th.row-seasonheader {
    background-color: transparent;
}

.sickchillTable {
    color: #000000;
}

.sickchillTable th {
    background-color: #15528F;
}

.sickchillTable th,
.sickchillTable td {
    border-top: 1px solid #222222;
    border-left: 1px solid #222222;
}

th.row-seasonheader {
    background-color: transparent;
    color: #FFFFFF;
}

/* =======================================================================
schedule.mako
========================================================================== */

h2.day, h2.network {
    background-color: #15528F;
}

.tvshowDiv {
    color: #000000;
}

.tvshowDiv a:hover {
    color: #09A2FF;
}

#showListTable td.tvShow a {
    color: #000000;
}

#showListTable td.tvShow a:hover {
    cursor: pointer;
    color: #09A2FF;
}

table.cal-odd {
    background-color: #333333;
}

table.cal-even {
    background-color: #3D3D3D;
}

.calendarShow .text .airtime {
    color: #FFFFFF
}

.calendarShow .text .episode-title {
    color: #AAAAAA
}

/* =======================================================================
config*.mako
========================================================================== */

.component-group {
    border-bottom: 1px dotted #555555;
}

.component-group-desc p {
    color: #DDDDDD;
}

#provider_order_list li,
#service_order_list li {
    background: #333333 !important;
    color: #FFFFFF;
}

#provider_order_list .ui-state-default.torrent-provider {
    background-color: #555555 !important;
}

#provider_order_list .ui-state-default.nzb-provider {
    background-color: #222222 !important;
}

/* =======================================================================
config_postProcessing.mako
========================================================================== */

#config div.example {
    background-color: #333333;
    border: 1px solid #111111;
}

.Key {
    background-color: #3D3D3D;
    border: 1px solid #111111;
}

.Key th, .tableHeader {
    background: #15528F;
}

.Key tr {
    border-bottom: 1px solid #111111;
}

.Key tr.even {
    background-color: #333333;
}

/* =======================================================================
config_notifications.mako
========================================================================== */

div.metadata_options {
    background: #333333;
    color: #FFFFFF;
    border: 1px solid #111111;
}

div.metadata_options label:hover {
    color: #FFFFFF;
    background-color: #15528F;
}

div.metadata_options label {
    color: #FFFFFF;
}

div.metadata_example label {
    color: #FFFFFF;
}

/* =======================================================================
manage*.mako
========================================================================== */

.separator {
    color: #FFFFFF;
}

/* =======================================================================
Global
========================================================================== */

span.path {
    color: #09A2FF;
    background-color: #333333;
}

/* =======================================================================
bootstrap Overrides
========================================================================== */

body {
    color: #FFFFFF;
    background-color: #222222;
}

pre {
     color: #FFFFFF;
     background-color: #3D3D3D;
     border-color: #111111;
 }

/* navbar styling */
.navbar-default {
    background-color: #15528F;
    filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#297AB8', endColorstr='#15528F');
    background: -webkit-gradient(linear, left top, left bottom, from(#297AB8), to(#15528F));
    background: -moz-linear-gradient(top, #297AB8, #15528F);
    border-color: #3E3F3A;
}

.navbar-default .navbar-nav > li > a:hover,
.navbar-default .navbar-nav > li > a:focus {
    background-color: #124477;
}

.navbar-default .navbar-nav > .active > a,
.navbar-default .navbar-nav > .active > a:hover,
.navbar-default .navbar-nav > .active > a:focus {
    color: #FFFFFF;
    background-color: #124477;
}

.navbar-default .navbar-toggle:hover,
.navbar-default .navbar-toggle:focus {
    background-color: #124477;
}

.navbar-default .navbar-toggle .icon-bar {
    background-color: #124477;
}

.navbar-default .navbar-nav > .open > a,
.navbar-default .navbar-nav > .open > a:hover,
.navbar-default .navbar-nav > .open > a:focus {
    background-color: #124477;
    color: #FFFFFF;
}

@media (max-width: 767px) {
    .navbar-default .navbar-nav .open .dropdown-menu > .active > a,
    .navbar-default .navbar-nav .open .dropdown-menu > .active > a:hover,
    .navbar-default .navbar-nav .open .dropdown-menu > .active > a:focus {
        color: #FFFFFF;
        background-color: #124477;
    }
}

.dropdown-menu > li > a:hover, .dropdown-menu > li > a:focus {
    color: #FFFFFF;
    text-decoration: none;
    background-color: #15528F;
}

.dropdown-menu > li > a {
    padding: 4px 36px 4px 20px;
    color: #FFFFFF;
}

.dropdown-menu {
    background-color: #333333;
    border: 1px solid rgba(0, 0, 0, 0.15);
    box-shadow: 0 6px 12px rgba(0, 0, 0, 0.176);
}

.btn {
    color: #FFFFFF;
    text-shadow: 0 1px 1px rgba(0, 0, 0, 0.75);
    background-color: #2672B6;
    *background-color: #2672B6;
    background-image: -ms-linear-gradient(top, #297AB8, #15528F);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#297AB8), to(#15528F));
    background-image: -webkit-linear-gradient(top, #297AB8, #15528F);
    background-image: -o-linear-gradient(top, #297AB8, #15528F);
    background-image: linear-gradient(top, #297AB8, #15528F);
    background-image: -moz-linear-gradient(top, #297AB8, #15528F);
    border: 1px solid #111111;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    border-color: #111111;
    filter: progid:dximagetransform.microsoft.gradient(startColorstr='#297AB8', endColorstr='#15528F', GradientType=0);
}

.btn:hover,
.btn:active,
.btn.active,
.btn.disabled,
.btn[disabled] {
    background-color: #2672B6;
    *background-color: #2672B6;
    color: #FFFFFF;
}

.btn:active,
.btn.active {
    color: #FFFFFF;
}

.btn:hover {
    color: #FFFFFF;
    text-decoration: none;
    *background-color: #2672B6;
    background: #2672B6 0 -150px;
    -webkit-transition: background-position 0.0s linear;
    -moz-transition: background-position 0.0s linear;
    -ms-transition: background-position 0.0s linear;
    -o-transition: background-position 0.0s linear;
    transition: background-position 0.0s linear;
}

.btn:focus {
    color: #FFFFFF;
}

.btn.active,
.btn:active {
    background-color: #2672B6 \9;
    background: #2672B6 none;
    color: #FFFFFF;
}

.btn.disabled,
.btn[disabled] {
    background: #15528F none;
}

@media (min-width: 768px) {
    .navbar .nav > li > .dropdown-menu:after {
        border-bottom: 6px solid #333333;
    }
}

/* =======================================================================
browser.css overrides
========================================================================== */

.fileBrowserDialog ul li {
    margin: 2px 0;
    list-style-type: none;
    cursor: pointer;
    background: #333333 !important;
}

.fileBrowserDialog ul li a:hover {
    color: #09A2FF;
    background: none !important;
}

/* =======================================================================
formWizard.css
========================================================================== */

legend.legendStep {
    color: #FFFFFF;
}

div.stepsguide .step p {
    border-bottom: 4px solid #23AFDC;
}

div.stepsguide .disabledstep p {
    border-bottom: 4px solid #1178B3;
}

div.formpaginate .prev, div.formpaginate .next {
    background: #2265A1;
}

/* =======================================================================
pnotify.css
========================================================================== */

.ui-pnotify-container {
    border: 1px solid #111111;
    background-image: -moz-linear-gradient(#333333, #3D3D3D) !important;
    background-image: linear-gradient(#333333, #3D3D3D) !important;
    background-image: -webkit-linear-gradient(#333333, #3D3D3D) !important;
    background-image: -o-linear-gradient(#333333, #3D3D3D) !important;
    filter: progid:dximagetransform.microsoft.gradient(startColorstr=#333333, endColorstr=#3D3D3D) !important;
    -ms-filter: progid:dximagetransform.microsoft.gradient(startColorstr=#333333, endColorstr=#3D3D3D) !important;
    -moz-box-shadow: 0 0 2px #000000;
    -webkit-box-shadow: 0 0 2px #000000;
    -o-box-shadow: 0 0 2px #000000;
    box-shadow: 0 0 2px #000000;
}

.ui-pnotify-title {
    color: #FFFFFF;
}

.ui-pnotify-text {
    color: #FFFFFF;
}

/* =======================================================================
tablesorter.css
========================================================================== */

.tablesorter {
    color: #FFFFFF;
    background-color: transparent;
}

.tablesorter th,
.tablesorter td {
    padding: 4px;
    border-top: #222222 1px solid;
    border-left: #222222 1px solid;
    vertical-align: middle;
}

/* add extra border at right edge */
.tablesorter th:last-child,
.tablesorter td:last-child {
    border-right: #222222 1px solid;
}

.tablesorter th {
    color: #FFFFFF;
    text-align: center;
    text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.3);
    background-color: #15528F;
    border-collapse: collapse;
    font-weight: normal;
}

.tablesorter thead .tablesorter-headerDesc {
    background: #297AB8 url(data:image/gif;base64,R0lGODlhFQAEAIAAAP///////yH5BAEAAAEALAAAAAAVAAQAAAINjB+gC+jP2ptn0WskLQA7) no-repeat center right;
}

.tablesorter thead .tablesorter-headerAsc {
    background: #297AB8 url(data:image/gif;base64,R0lGODlhFQAEAIAAAP///////yH5BAEAAAEALAAAAAAVAAQAAAINjI8Bya2wnINUMopZAQA7) no-repeat center right;
}

thead.tablesorter-stickyHeader {
    border-top: 2px solid #222222;
    border-bottom: 2px solid #222222;
}

/* Zebra Widget - row alternating colors */
.tablesorter tr.odd, .sickchillTable tr.odd {
    background-color: #333333;
}

.tablesorter tr.even, .sickchillTable tr.even {
    background-color: #2E2E2E;
}

.tablesorter tr.tablesorter-filter-row,
.tablesorter tr.tablesorter-filter-row td {
    text-align: center;
    background: #333333;
    border-bottom: 1px solid #111111;
}

/* add extra border at left edge */
.tablesorter tr.tablesorter-filter-row td:first-child {
    border-left: #222222 1px solid;
}

/* add extra border at right edge */
.tablesorter tr.tablesorter-filter-row td:last-child {
    border-right: #222222 1px solid;
}

#showListTable tbody {
    color: #000000;
}

/* =======================================================================
token-input.css
========================================================================== */

div.token-input-dropdown {
    background-color: #FFFFFF;
    color: #000000;
    border-left-color: #CCCCCC;
    border-right-color: #CCCCCC;
    border-bottom-color: #CCCCCC;
}

/* =======================================================================
jquery.confirm.css
========================================================================== */

#confirmBox {
    background: #222222;
}

#confirmBox h1 {
    background-color: #15528F;
}

#confirmBox p {
    color: #FFFFFF;
}

/* =======================================================================
bootstrap modal
========================================================================== */

.modal-content {
    background-color: #3D3D3D;
}

.modal-body {
    background-color: #3D3D3D;
}

.modal-header {
    background-color: #15528F;
}

/* =======================================================================
bootstrap panel
========================================================================== */

.panel-default {
    background-color: #3D3D3D;
    border-color: #111111;
}

.panel-heading {
    background-color: #3D3D3D !important;
    color: #FFFFFF !important;
}

/* =======================================================================
new #confirmBox
========================================================================== */
.modal-dialog {
    position: fixed;
    text-align: center;
    width: 100%;
    top: 30%;
    margin: 0 !important;
}

.modal-content {
    max-width: 460px;
    margin: 10px;
    box-shadow: 0 0 12px 0 rgba(0, 0, 0, 0.175);
    border: 1px solid #111111;
    display: inline-block;
    border-radius: 0;
}

.modal-header {
    border-bottom: none;
    border-radius: 0;
}

.modal-body, .modal-content {
    background: #222222;
}

.modal-footer {
    border-top: none;
}

.confirmation-modal {
    background: -moz-linear-gradient(rgba(0, 0, 0, 0.5), rgba(0, 0, 0, 0.5)) repeat-x rgba(0, 0, 0, 0.5);
    background: -webkit-gradient(linear, 0% 0%, 0% 100%, from(rgba(0, 0, 0, 0.5)), to(rgba(0, 0, 0, 0.5))) repeat-x rgba(0, 0, 0, 0.5);
}

.modal-footer {
    text-align: center;
}

.modal-header .close {
    display: none
}

.modal-footer button {
    margin-right: 15px;
    padding: 2px 15px;
    text-decoration: none;
    display: inline-block;
    color: #FFFFFF;
    text-align: center;
    text-shadow: 0 1px 1px rgba(0, 0, 0, 0.75);
    background-clip: padding-box;
    border: 1px solid #111111;
    border-radius: 3px;
    cursor: pointer;
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
    background-image: -webkit-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    background-image: -moz-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    background-image: -o-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    background-image: linear-gradient(to bottom, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, rgba(0, 0, 0, 0) 51%, rgba(0, 0, 0, 0.25));
    -webkit-box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
    box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
}

.modal-footer button:last-child {
    margin-right: 0;
}

.modal-footer button.confirm {
    background-color: #3F7636;
}

.modal-footer button.confirm:hover {
    background-color: #48873E;
}

.modal-footer button.cancel {
    background-color: #8D2D2B;
}

.modal-footer button.cancel:hover {
    background-color: #A13331;
}
