#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["just now", "right now"],
    ["%ss ago", "in %ss"],
    ["1m ago", "in 1m"],
    ["%sm ago", "in %sm"],
    ["1h ago", "in 1h"],
    ["%sh ago", "in %sh"],
    ["1d ago", "in 1d"],
    ["%sd ago", "in %sd"],
    ["1w ago", "in 1w"],
    ["%sw ago", "in %sw"],
    ["1mo ago", "in 1mo"],
    ["%smo ago", "in %smo"],
    ["1yr ago", "in 1yr"],
    ["%syr ago", "in %syr"]
]
        